@@ -20,7 +20,7 @@ XS_STRING = "xs:string"
 XS_DOUBLE = "xs:double"
 XS_BOOL = "xs:boolean"
 XS_INT = "xs:int"
-
+TEMPLATE_NAME = "template.txt"
 RANGE_STRINGS = ["Range", "range", "RANGE"]
 
 VALUE_TYPE_LOOKUP = {VALUE: "Value", FORMAT: "Format", RANGE: "Range", LIST: "List", }
@@ -36,7 +36,11 @@ FILEPATH_JS = "js_templates"
 IGNORE_PSET = "IFC"
 INHERITED_TEXT = "Predefined Pset"
 
-# xml export
+# json export
+CURRENT_PR0JECT_PHASE = "current_project_phase"
+PROJECT_PHASES = "ProjectPhases"
+AGGREGATION_PSET = "AggregationPsetName"
+AGGREGATION_ATTRIBUTE = "AggregationAttributeName"
 PREDEFINED_PSETS = "PredefinedPropertySets"
 PREDEFINED_PSET = "PredefinedPropertySet"
 PROPERTY_SET = "PropertySet"
